package common.util.file;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;

import common.exception.AppException;

/**
 * ファイル入出力処理
 * @author ishihara
 *
 */
public class FileManager {

	private static final int NOT_OPEN = 0;
	private static final int READ_OPEN = 1;
	private static final int WRITE_OPEN = 2;

	/** リード用 */
	private FileInputStream fis ;
	private InputStreamReader isr ;
	private BufferedReader reader ;

	/** ライト用 */
	private FileOutputStream fos ;
	private OutputStreamWriter osw ;
	private BufferedWriter writer ;

	/** オープン種別 */
	private int openType = NOT_OPEN ;
	/**  オープンファイルパス */
	private String openFilePath = null ;
	/**   */
	private String logmsg ;

	private static final String FILE_NOT_OPEN = "ファイルがオープンされていない。";

	/** コンストラクタ */
	public FileManager(){
		openType = NOT_OPEN ;
		openFilePath = null ;
	}
	/**
	 * ランダムファイルオープン
	 * @param fpath
	 * @param mode
	 *  "r" 	読み込み用に開く。開いたオブジェクトの任意の write メソッドを呼び出すと、IOException がスローされる
	 *  "rw" 	読み込みおよび書き込み用に開く。ファイルが存在しない場合は、作成しようとする
	 * @return RandomAccessFileオブジェクト
	 * @throws AppException
	 */
	public RandomAccessFile randomOpen(String fpath, String mode) throws AppException{

		RandomAccessFile file= null ;
		try {
			file = new RandomAccessFile(fpath, mode);
		} catch (FileNotFoundException e) {
			throw new AppException(null, e);
		}
		return file ;
	}

	/**
	 * ランダムファイルクローズ
	 * @param file
	 * @throws AppException
	 */
	public void randomClose(RandomAccessFile file) throws AppException{
		try {
			file.close();
		} catch (IOException e) {
			throw new AppException(null, e);
		}
	}
	/**
	 * 指定された行番号の行を読み込む
	 * @param fpath ファイルパス
	 * @param mode オープンモード
	 * @param lineNo 行番号
	 * @return
	 * @throws AppException
	 */
	public String readRandomFile(RandomAccessFile file, long lineNo) throws AppException{
		String line = null ;

		try {
//			RandomAccessFile file = new RandomAccessFile(fpath, mode);

			file.seek(0L);

			// (lineNo-1)行スキップする
			long i=1;
			for(; i < lineNo; i++){
				line = file.readLine();
				if (line == null){
					//行数不足
					throw new AppException("行数不足。ファイル行数＝"+ i, (Exception)null);
				}
			}
			line = file.readLine();
			if (line == null){
				//行数不足
				throw new AppException("行数不足。ファイル行数＝"+ i, (Exception)null);
			}
		} catch (FileNotFoundException e) {
			throw new AppException(e);
		} catch (IOException e) {
			throw new AppException(e);
		}
		return line ;
	}

	/**
	 * 読み込みオープン
	 * @param fpath ファイルパス
	 * @param encode 文字コード
	 * @throws AppException FileNotFoundException、UnsupportedEncodingException
	 */
	public void readOpen(String fpath, String encode) throws AppException{
		openType = NOT_OPEN ;

		try {
			fis = new FileInputStream(fpath);
			isr = new InputStreamReader(fis, encode);
			reader = new BufferedReader(isr);

			openType = READ_OPEN ;
			openFilePath = fpath ;

		} catch (FileNotFoundException e) {
			throw new AppException(e);
		} catch (UnsupportedEncodingException e) {
			throw new AppException(e);
		}
	}
	/**
	 * 書込みオープン
	 * @param fpath ファイルパス
	 * @param encode 文字コード
	 * @throws AppException FileNotFoundException、UnsupportedEncodingException
	 */
	public void writeOpen(String fpath, String encode, boolean append) throws AppException{
		openType = NOT_OPEN ;

		try {
			fos = new FileOutputStream(fpath, append);
			osw = new OutputStreamWriter(fos, encode);
			writer = new BufferedWriter(osw);

			openType = WRITE_OPEN ;
			openFilePath = fpath ;

		} catch (FileNotFoundException e) {
			throw new AppException(e);
		} catch (UnsupportedEncodingException e) {
			throw new AppException(e);
		}
	}
	/**
	 * ファイル読み込み（１レコード）
	 * @return 読み込んだ内容　または　null
	 * @throws AppException ファイル未オープン, IOException
	 */
	public String readFile() throws AppException{

		String line = null  ;

		if ((openType == NOT_OPEN) || (openFilePath == null) || (reader == null)){
			throw new AppException(FILE_NOT_OPEN , (Exception)null);
		}
		try {
			while ((line = reader.readLine()) != null) {
				break;
			}
		} catch (IOException e) {
			throw new AppException((String)null, e);
		}
		return line ;
	}
	/**
	 * ファイル書込み
	 *
	 * @param buff 保存するテキスト
	 * @throws AppException
	 * @throws IOException ファイルIO例外
	 */
	public void writeFile(final String buff, final boolean append,
			final String encode) throws AppException  {

		if ((openType == NOT_OPEN) || (openFilePath == null) || (writer == null)){
			throw new AppException(FILE_NOT_OPEN , (Exception)null);
		}

		try {
			writer.write(buff);
		} catch (IOException e) {
			throw new AppException((String)null, e);
		}

		return;
	}

	/**
	 * ファイルクローズ
	 * @throws AppException ファイル未オープン,IOException
	 */
	public void closeFile() throws AppException{

		switch(openType){
		case READ_OPEN:
			if (reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					throw new AppException((String)null, e);
				}
			}
			if (isr != null){
				try {
					isr.close();
				} catch (IOException e) {
					throw new AppException((String)null, e);
				}
			}
			if (fis != null){
				try {
					fis.close();
				} catch (IOException e) {
					throw new AppException((String)null, e);
				}
			}
			openType = NOT_OPEN ;
			openFilePath = null ;
			break;
		case WRITE_OPEN:
			if (writer != null){
				try {
					writer.close();
				} catch (IOException e) {
					throw new AppException((String)null,e);
				}
			}
			if (osw != null){
				try {
					osw.close();
				} catch (IOException e) {
					throw new AppException((String)null, e);
				}
			}
			if (fos != null){
				try {
					fos.close();
				} catch (IOException e) {
					throw new AppException((String)null, e);
				}
			}
			openType = NOT_OPEN ;
			openFilePath = null ;
			break;
		case NOT_OPEN:
			throw new AppException("ファイルがオープンされていない", (Exception)null);

		}
	}
}
